@@ -7,6 +7,8 @@
 from __future__ import absolute_import, print_function
 
 # Python3-friendly imports
+import os
+
 try:
     from urllib.parse import urlparse
 except ImportError:
@@ -14,17 +16,21 @@ except ImportError:
 
 import json
 import logging
+import re
 
 import tornado.web
 import tornado.websocket
 
+
 def _cast_unicode(s):
     if isinstance(s, bytes):
         return s.decode('utf-8')
     return s
 
+
 class TermSocket(tornado.websocket.WebSocketHandler):
     """Handler for a terminal websocket"""
+
     def initialize(self, term_manager):
         self.term_manager = term_manager
         self.term_name = ""
@@ -32,6 +38,7 @@ class TermSocket(tornado.websocket.WebSocketHandler):
         self.terminal = None
 
         self._logger = logging.getLogger(__name__)
+        self._user_command = ''
 
     def origin_check(self, origin=None):
         """Deprecated: backward-compat for terminado <= 0.5."""
@@ -39,7 +46,7 @@ class TermSocket(tornado.websocket.WebSocketHandler):
 
     def open(self, url_component=None):
         """Websocket connection opened.
-        
+
         Call our terminal manager to get a terminal, and connect to it as a
         client.
         """
@@ -52,12 +59,18 @@ class TermSocket(tornado.websocket.WebSocketHandler):
         url_component = _cast_unicode(url_component)
         self.term_name = url_component or 'tty'
         self.terminal = self.term_manager.get_terminal(url_component)
-        for s in self.terminal.read_buffer:
-            self.on_pty_read(s)
         self.terminal.clients.append(self)
-
         self.send_json_message(["setup", {}])
         self._logger.info("TermSocket.open: Opened %s", self.term_name)
+        # Now drain the preopen buffer, if it exists.
+        buffered = ""
+        while True:
+            if not self.terminal.preopen_buffer:
+                break
+            s = self.terminal.preopen_buffer.popleft()
+            buffered += s
+        if buffered:
+            self.on_pty_read(buffered)
 
     def on_pty_read(self, text):
         """Data read from pty; send to frontend"""
@@ -65,27 +78,38 @@ class TermSocket(tornado.websocket.WebSocketHandler):
 
     def send_json_message(self, content):
         json_msg = json.dumps(content)
+        pattern = re.compile(r'^(\w|\d)+')
+        try:
+            if pattern.search(content[1]):
+                self.log_terminal_output(f'STDOUT: {content[1]}')
+        except TypeError as e:
+            self._logger.error(f'not able to serialize: {e}')
         self.write_message(json_msg)
 
     def on_message(self, message):
         """Handle incoming websocket message
-        
+
         We send JSON arrays, where the first element is a string indicating
         what kind of message this is. Data associated with the message follows.
         """
         ##logging.info("TermSocket.on_message: %s - (%s) %s", self.term_name, type(message), len(message) if isinstance(message, bytes) else message[:250])
         command = json.loads(message)
-        msg_type = command[0]    
+        msg_type = command[0]
 
         if msg_type == "stdin":
             self.terminal.ptyproc.write(command[1])
+            if command[1] == '\r':
+                self.log_terminal_output(f'STDIN: {self._user_command}')
+                self._user_command = ''
+            else:
+                self._user_command += command[1]
         elif msg_type == "set_size":
             self.size = command[1:3]
             self.terminal.resize_to_smallest()
 
     def on_close(self):
         """Handle websocket closing.
-        
+
         Disconnect from our terminal, and tell the terminal manager we're
         disconnecting.
         """
@@ -100,3 +124,13 @@ class TermSocket(tornado.websocket.WebSocketHandler):
         """
         self.send_json_message(['disconnect', 1])
         self.close()
+        self.terminal = None
+
+    def log_terminal_output(self, log: str = ''):
+        """
+        Logs the terminal input/output if the environment variable LOG_TERMINAL_OUTPUT is "true"
+        :param log: log line to write
+        :return:
+        """
+        if str.lower(os.getenv("LOG_TERMINAL_OUTPUT", "false")) == "true":
+            self._logger.debug(log)
